# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(name='video_brain',
      version='0.1',
      description='Deep Learning algorithms for video processing tasks',
      # url='http://github.com/storborg/funniest',
      author='Léo Vetter',
      author_email='vetter.leo@gmail.com',
      license='MIT',
      packages=find_packages(),
      zip_safe=False,
      test_suite='nose.collector',
      tests_require=['nose'],
      include_package_data=True,
      )